@@ -411,7 +411,8 @@ def _custom_jvp_call_mlir_translation(ctx, *args, call_jaxpr, jvp_jaxpr_thunk,
   args_ = map(mlir.wrap_singleton_ir_values, args)
   consts = mlir._ir_consts(call_jaxpr.consts)
   out, tokens = mlir.jaxpr_subcomp(ctx.module_context, call_jaxpr.jaxpr,
-                                   ctx.tokens_in, consts, *args_)
+                                   ctx.tokens_in, consts, *args_,
+                                   dim_var_values=ctx.dim_var_values)
   ctx.set_tokens_out(tokens)
   return out
 mlir.register_lowering(custom_jvp_call_p, _custom_jvp_call_mlir_translation)
@@ -1066,13 +1067,13 @@ def linear_call(fun: Callable, fun_transpose: Callable, residual_args,
   ...   return transposed
 
   >>> div_add(9., 3.)
-  DeviceArray(12., dtype=float32, weak_type=True)
+  Array(12., dtype=float32, weak_type=True)
 
   >>> transpose(partial(div_add, denom=3.), 1.)(18.)  # custom
-  DeviceArray(24., dtype=float32, weak_type=True)
+  Array(24., dtype=float32, weak_type=True)
 
   >>> transpose(lambda x: x + x / 3., 1.)(18.)  # reference
-  DeviceArray(24., dtype=float32, weak_type=True)
+  Array(24., dtype=float32, weak_type=True)
 
   The above definition of ``f`` illustrates the purpose of a residual
   argument: division is linear in one of its inputs (the dividend
