@@ -90,7 +90,7 @@ def switch(index, branches: Sequence[Callable], *operands,
   if operand is not _no_operand_sentinel:
     if operands:
       raise TypeError("if 'operand' keyword is passed then no positional "
-                      f"operands can be passed, got operand={operand} "
+                      f"operands can be passed, got {operand=} "
                       f"and positional operands {operands}")
     operands = (operand,)
   del operand
@@ -184,7 +184,7 @@ def _cond(pred, true_fun: Callable, false_fun: Callable, *operands,
   if operand is not _no_operand_sentinel:
     if operands:
       raise TypeError("if 'operand' keyword is passed then no positional "
-                      f"operands can be passed, got operand={operand} "
+                      f"operands can be passed, got {operand=} "
                       f"and positional operands {operands}")
     operands = (operand,)
   del operand
@@ -246,6 +246,10 @@ def _cond(pred, true_fun: Callable, false_fun: Callable, *operands,
     # Raise index in case of effects to allow data-dependence-based discharging
     # of those effects (even if they don't have an explicit data dependence).
     index = core.raise_as_much_as_possible(index)
+  false_jaxpr = false_jaxpr.replace(
+      jaxpr=false_jaxpr.jaxpr.replace(effects=joined_effects))
+  true_jaxpr = true_jaxpr.replace(
+      jaxpr=true_jaxpr.jaxpr.replace(effects=joined_effects))
 
   linear = [False] * len(consts) + linear_ops
   out = cond_p.bind(
@@ -816,7 +820,8 @@ def _cond_lowering(ctx, index, *args, branches, linear):
       out_vals, tokens_out = mlir.jaxpr_subcomp(
           sub_ctx, jaxpr.jaxpr, tokens_in,
           map(mlir.ir_constants, jaxpr.consts),
-          *map(mlir.wrap_singleton_ir_values, args))
+          *map(mlir.wrap_singleton_ir_values, args),
+          dim_var_values=ctx.dim_var_values)
       out_tokens = [tokens_out.get(eff) for eff in ordered_effects]
       out_vals = [*out_tokens, *out_vals]
       mhlo.ReturnOp(util.flatten(out_vals))
