@@ -20,6 +20,7 @@ import warnings
 
 import numpy as np
 
+import jax
 from jax import lax
 from jax import core
 from jax import numpy as jnp
@@ -586,7 +587,7 @@ def multivariate_normal(key: KeyArray,
                         mean: RealArray,
                         cov: RealArray,
                         shape: Optional[Shape] = None,
-                        dtype: DTypeLikeFloat = dtypes.float_,
+                        dtype: DTypeLikeFloat = None,
                         method: str = 'cholesky') -> Array:
   """Sample multivariate normal random values with given mean and covariance.
 
@@ -603,19 +604,22 @@ def multivariate_normal(key: KeyArray,
     dtype: optional, a float dtype for the returned values (default float64 if
       jax_enable_x64 is true, otherwise float32).
     method: optional, a method to compute the factor of ``cov``.
-      Must be one of 'svd', eigh, and 'cholesky'. Default 'cholesky'.
+      Must be one of 'svd', 'eigh', and 'cholesky'. Default 'cholesky'. For
+      singular covariance matrices, use 'svd' or 'eigh'.
   Returns:
     A random array with the specified dtype and shape given by
     ``shape + mean.shape[-1:]`` if ``shape`` is not None, or else
     ``broadcast_shapes(mean.shape[:-1], cov.shape[:-2]) + mean.shape[-1:]``.
   """
   key, _ = _check_prng_key(key)
+  mean, cov = _promote_dtypes_inexact(mean, cov)
   if method not in {'svd', 'eigh', 'cholesky'}:
     raise ValueError("method must be one of {'svd', 'eigh', 'cholesky'}")
+  if dtype is None:
+    dtype = mean.dtype
   if not dtypes.issubdtype(dtype, np.floating):
     raise ValueError(f"dtype argument to `multivariate_normal` must be a float "
                      f"dtype, got {dtype}")
-  dtype = dtypes.canonicalize_dtype(dtype)
   if shape is not None:
     shape = core.canonicalize_shape(shape)
   return _multivariate_normal(key, mean, cov, shape, dtype, method)  # type: ignore
@@ -648,7 +652,9 @@ def _multivariate_normal(key, mean, cov, shape, dtype, method) -> Array:
   else: # 'cholesky'
     factor = cholesky(cov)
   normal_samples = normal(key, shape + mean.shape[-1:], dtype)
-  return mean + jnp.einsum('...ij,...j->...i', factor, normal_samples)
+  with jax.numpy_rank_promotion('allow'):
+    result = mean + jnp.einsum('...ij,...j->...i', factor, normal_samples)
+  return result
 
 
 def truncated_normal(key: KeyArray,
