@@ -33,8 +33,7 @@ from __future__ import annotations
 import warnings
 
 from dataclasses import dataclass
-from typing import Any, Callable, Dict, List, NamedTuple, Optional, Sequence, Tuple
-from typing_extensions import Protocol
+from typing import Any, Dict, List, NamedTuple, Optional, Protocol, Sequence, Tuple
 
 import jax
 from jax import core
@@ -139,6 +138,10 @@ class Executable(Protocol):
     """
     raise NotImplementedError
 
+  def create_cpp_call(self, no_kwargs, in_tree, out_tree) -> Any:
+    """Optionally constructs a fast c++ dispatcher."""
+    return None
+
 
 class Lowering(Protocol):
   """Protocol for lowerings, which a user-facing ``Lowered`` encapsulates."""
@@ -208,17 +211,30 @@ class XlaExecutable(Executable):
     xla_ext_exe = self.xla_extension_executable()
     err_msg = ("cost analysis unsupported on current XLA backend: "
                f"{type(xla_ext_exe)}")
-    if not hasattr(xla_ext_exe, "client"):
+    # TODO(b/259255524): Unify/merge the two cost_analysis calls below.
+    if hasattr(xla_ext_exe, "client"):
+      try:
+        return [
+            xla_extension.hlo_module_cost_analysis(xla_ext_exe.client, m)
+            for m in xla_ext_exe.hlo_modules()
+        ]
+      except xla_extension.XlaRuntimeError as e:
+        msg, *_ = e.args
+        if type(msg) is str and msg.startswith("UNIMPLEMENTED"):
+          raise NotImplementedError(err_msg) from e
+        else:
+          raise
+    elif hasattr(xla_ext_exe, "cost_analysis"):
+      try:
+        return xla_ext_exe.cost_analysis()
+      except xla_extension.XlaRuntimeError as e:
+        msg, *_ = e.args
+        if type(msg) is str and msg.startswith("UNIMPLEMENTED"):
+          raise NotImplementedError(err_msg) from e
+        else:
+          raise
+    else:
       raise NotImplementedError(err_msg)
-    try:
-      return [xla_extension.hlo_module_cost_analysis(xla_ext_exe.client, m)
-              for m in xla_ext_exe.hlo_modules()]
-    except xla_extension.XlaRuntimeError as e:
-      msg, *_ = e.args
-      if type(msg) is str and msg.startswith("UNIMPLEMENTED"):
-        raise NotImplementedError(err_msg) from e
-      else:
-        raise
 
   def memory_analysis(self) -> Any:
     xla_ext_exe = self.xla_extension_executable()
@@ -342,19 +358,16 @@ class Compiled(Stage):
   _executable: Executable
   _no_kwargs: bool
 
-  def __init__(self, executable, args_info, out_tree, no_kwargs=False, create_cpp_call=None):
+  def __init__(self, executable, args_info, out_tree, no_kwargs=False):
     self._executable = executable
     self._no_kwargs = no_kwargs
     self.args_info = args_info
     self.out_tree = out_tree
     self._params = CompiledCallParams(self._executable, self._no_kwargs,
                                       self.in_tree, self.out_tree)
-    # TODO(chky): Remove this conditional statement once we implement the fast
-    # path in C++ for all AOT paths.
-    if create_cpp_call is not None:
-      self._cpp_call = create_cpp_call(self._params)
-    else:
-      self._cpp_call = None
+    self._cpp_call = self._executable.create_cpp_call(self._no_kwargs,
+                                                      self.in_tree,
+                                                      self.out_tree)
 
   def compiler_ir(self):
     """Post-compilation IR.
@@ -445,6 +458,10 @@ class Compiled(Stage):
 
   @staticmethod
   def call(*args, **kwargs):
+    # This is because `__call__` passes in `self._params` as the first argument.
+    # Instead of making the call signature `call(params, *args, **kwargs)`
+    # extract it from args because `params` can be passed as a kwarg by users
+    # which might confict here.
     params = args[0]
     args = args[1:]
     if jax.config.jax_dynamic_shapes:
@@ -505,18 +522,17 @@ class Lowered(Stage):
   out_tree: tree_util.PyTreeDef
   _lowering: XlaLowering
   _no_kwargs: bool
-  _create_cpp_call: Optional[Callable]
 
-  def __init__(self,
-               lowering: XlaLowering,
-               args_info,       # PyTreee of ArgInfo
-               out_tree: tree_util.PyTreeDef,
-               no_kwargs: bool = False, create_cpp_call: Optional[Callable] = None):
+  def __init__(
+      self,
+      lowering: XlaLowering,
+      args_info,  # PyTreee of ArgInfo
+      out_tree: tree_util.PyTreeDef,
+      no_kwargs: bool = False):
     self._lowering = lowering
     self._no_kwargs = no_kwargs
     self.args_info = args_info
     self.out_tree = out_tree
-    self._create_cpp_call = create_cpp_call
 
   @classmethod
   def from_flat_info(cls,
@@ -525,7 +541,7 @@ class Lowered(Stage):
                      in_avals,
                      donate_argnums: Tuple[int, ...],
                      out_tree: tree_util.PyTreeDef,
-                     no_kwargs: bool = False, create_cpp_call: Optional[Callable] = None):
+                     no_kwargs: bool = False):
     """Initialize from flat info (``in_avals`` etc.) and an input PyTreeDef.
 
     Args:
@@ -535,8 +551,11 @@ class Lowered(Stage):
         ``Compiled`` returned from this object will not support keyword
         arguments (an error will be raised if some are provided).
     """
-    return cls(lowering, make_args_info(in_tree, in_avals, donate_argnums),
-               out_tree, no_kwargs=no_kwargs, create_cpp_call=create_cpp_call)
+    return cls(
+        lowering,
+        make_args_info(in_tree, in_avals, donate_argnums),
+        out_tree,
+        no_kwargs=no_kwargs)
 
   def compile(self) -> Compiled:
     """Compile, returning a corresponding ``Compiled`` instance."""
@@ -549,8 +568,11 @@ class Lowered(Stage):
     else:
       kw = {}
 
-    return Compiled(self._lowering.compile(**kw), self.args_info, self.out_tree,
-                    no_kwargs=self._no_kwargs, create_cpp_call=self._create_cpp_call)
+    return Compiled(
+        self._lowering.compile(**kw),
+        self.args_info,
+        self.out_tree,
+        no_kwargs=self._no_kwargs)
 
   def as_text(self, dialect: Optional[str] = None) -> str:
     """A human-readable text representation of this lowering.
