@@ -46,7 +46,6 @@ from jax._src.public_test_util import (  # noqa: F401
     _assert_numpy_allclose, _check_dtypes_match, _default_tolerance, _dtype, check_close, check_grads,
     check_jvp, check_vjp, default_gradient_tolerance, default_tolerance, device_under_test, tolerance)
 from jax.interpreters import mlir
-from jax.experimental.maps import Mesh
 
 # This submodule includes private test utilities that are not exported to
 # jax.test_util. Functionality appearing here is for internal use only, and
@@ -303,6 +302,20 @@ def set_host_platform_device_count(nr_devices: int):
     xla_bridge.get_backend.cache_clear()
   return undo
 
+
+def skip_on_xla_cpu_mlir(test_method):
+  """A decorator to skip tests when MLIR lowering is enabled."""
+  @functools.wraps(test_method)
+  def test_method_wrapper(self, *args, **kwargs):
+    xla_flags = os.getenv('XLA_FLAGS') or ''
+    if '--xla_cpu_use_xla_runtime' in xla_flags or '--xla_cpu_enable_mlir_lowering' in xla_flags:
+      test_name = getattr(test_method, '__name__', '[unknown test]')
+      raise unittest.SkipTest(
+          f'{test_name} not supported on XLA:CPU MLIR')
+    return test_method(self, *args, **kwargs)
+  return test_method_wrapper
+
+
 def skip_on_flag(flag_name, skip_value):
   """A decorator for test methods to skip the test when flags are set."""
   def skip(test_method):        # pylint: disable=missing-docstring
@@ -986,7 +999,7 @@ def with_mesh(named_shape: MeshSpec) -> Generator[None, None, None]:
   if len(local_devices) < size:
     raise unittest.SkipTest(f"Test requires {size} local devices")
   mesh_devices = np.array(local_devices[:size]).reshape(shape)  # type: ignore
-  with Mesh(mesh_devices, axis_names):
+  with jax.sharding.Mesh(mesh_devices, axis_names):
     yield
 
 def with_mesh_from_kwargs(f):
@@ -1026,7 +1039,7 @@ def create_global_mesh(mesh_shape, axis_names):
     raise unittest.SkipTest(f"Test requires {size} global devices.")
   devices = sorted(api.devices(), key=lambda d: d.id)
   mesh_devices = np.array(devices[:size]).reshape(mesh_shape)
-  global_mesh = Mesh(mesh_devices, axis_names)
+  global_mesh = jax.sharding.Mesh(mesh_devices, axis_names)
   return global_mesh
 
 
@@ -1114,3 +1127,13 @@ def strict_promotion_if_dtypes_match(dtypes):
   if all(dtype == dtypes[0] for dtype in dtypes):
     return jax.numpy_dtype_promotion('strict')
   return jax.numpy_dtype_promotion('standard')
+
+_version_regex = re.compile(r"([0-9]+(?:\.[0-9]+)*)(?:(rc|dev).*)?")
+def _parse_version(v: str) -> Tuple[int, ...]:
+  m = _version_regex.match(v)
+  if m is None:
+    raise ValueError(f"Unable to parse version '{v}'")
+  return tuple(int(x) for x in m.group(1).split('.'))
+
+def numpy_version():
+  return _parse_version(np.__version__)
