@@ -111,6 +111,8 @@ class FrozenDict(abc.Mapping):
 
 AxisName = core.AxisName
 ResourceAxisName = AxisName  # Different name just for documentation purposes
+# TODO(https://github.com/google/jax/issues/13487): Remove Mesh in
+# 3 months from `jax.experimental.maps.Mesh`.
 Mesh = pxla.Mesh
 ResourceEnv = pxla.ResourceEnv
 EMPTY_ENV = pxla.EMPTY_ENV
@@ -415,8 +417,8 @@ def xmap(fun: Callable,
   >>> xmap(jnp.vdot,
   ...      in_axes=({0: 'left'}, {1: 'right'}),
   ...      out_axes=['left', 'right', ...])(x, x.T)
-  DeviceArray([[ 30,  80],
-               [ 80, 255]], dtype=int32)
+  Array([[ 30,  80],
+         [ 80, 255]], dtype=int32)
 
   Note that the contraction in the program is performed over the positional axes,
   while named axes are just a convenient way to achieve batching. While this
@@ -677,7 +679,8 @@ def make_xmap_callable(fun: lu.WrappedFun,
                      for aval, in_axes in zip(in_avals, in_axes)]
   with core.extend_axis_env_nd(global_axis_sizes.items()):
     with dispatch.log_elapsed_time(f"Finished tracing + transforming {fun.__name__} "
-                                   "for xmap in {elapsed_time} sec"):
+                                   "for xmap in {elapsed_time} sec",
+                                    event=dispatch.JAXPR_TRACE_EVENT):
       jaxpr, out_avals, consts = pe.trace_to_jaxpr_final(fun, mapped_in_avals)
   out_axes = out_axes_thunk()
   _check_out_avals_vs_out_axes(out_avals, out_axes, global_axis_sizes)
@@ -1390,7 +1393,8 @@ def _xmap_lowering_rule_replica(ctx, *in_nodes,
   if any(eff in core.ordered_effects for eff in vectorized_jaxpr.effects):
     raise NotImplementedError('Cannot lower `xmap` with ordered effects.')
   tiled_outs, _ = mlir.jaxpr_subcomp(sub_ctx, vectorized_jaxpr, mlir.TokenSet(),
-                                     const_nodes, *tiled_ins)
+                                     const_nodes, *tiled_ins,
+                                     dim_var_values=ctx.dim_var_values)
 
   outs = [
       mlir.lower_fun(
@@ -1456,7 +1460,8 @@ def _xmap_lowering_rule_spmd(ctx, *global_in_nodes,
   if any(eff in core.ordered_effects for eff in vectorized_jaxpr.effects):
     raise NotImplementedError('Cannot lower `xmap` with ordered effects.')
   global_out_nodes, _ = mlir.jaxpr_subcomp(sub_ctx, vectorized_jaxpr,
-      mlir.TokenSet(), const_nodes, *sharded_global_in_nodes)
+      mlir.TokenSet(), const_nodes, *sharded_global_in_nodes,
+      dim_var_values=ctx.dim_var_values)
 
   sharded_global_out_nodes = [
     mlir.wrap_with_sharding_op(node, global_sharding_spec(aval, aval_axes).sharding_proto())
@@ -1507,7 +1512,8 @@ def _xmap_lowering_rule_spmd_manual(ctx, *global_in_nodes,
   if any(eff in core.ordered_effects for eff in vectorized_jaxpr.effects):
     raise NotImplementedError('Cannot lower `xmap` with ordered effects.')
   global_out_nodes, _ = mlir.jaxpr_subcomp(sub_ctx, vectorized_jaxpr,
-      mlir.TokenSet(), const_nodes, *([n] for n in global_in_nodes))
+      mlir.TokenSet(), const_nodes, *([n] for n in global_in_nodes),
+      dim_var_values=ctx.dim_var_values)
 
   return global_out_nodes
 
