@@ -49,6 +49,7 @@ from jax.interpreters import partial_eval as pe
 from jax.interpreters.pxla import PartitionSpec
 from jax._src.lib.mlir import ir
 from jax._src.lib.mlir.dialects import func as func_dialect
+from jax._src.lib import xla_bridge as xb
 from jax._src.lib import xla_client as xc
 from jax._src.lib import xla_extension_version
 from jax.tree_util import (tree_map, tree_flatten, tree_unflatten,
@@ -120,14 +121,6 @@ def _python_pjit(fun: Callable, infer_params):
 
   return wrapped
 
-class _PjitFastpathData(NamedTuple):
-  xla_executable: xla.XlaLoadedExecutable
-  out_pytree_def: Any
-  in_shardings: Sequence[Any]
-  out_shardings: Sequence[Any]
-  out_avals: Sequence[Any]
-  out_committed: Sequence[bool]
-
 class _MostRecentPjitCallExecutable(threading.local):
   def __init__(self):
     self.value = None
@@ -156,58 +149,24 @@ def _cpp_pjit(fun: Callable, infer_params, static_argnums):
     if use_fastpath:
       out_avals = [o.aval for o in out_flat]
       out_committed = [o._committed for o in out_flat]
-      fastpath_data = _PjitFastpathData(executable.xla_executable,
-                                        out_tree,
-                                        executable._in_shardings,
-                                        executable._out_shardings, out_avals,
-                                        out_committed)
+      fastpath_data = pxla._MeshExecutableFastpathData(
+          executable.xla_executable, out_tree, executable._in_shardings,
+          executable._out_shardings, out_avals, out_committed)
     else:
       fastpath_data = None
 
 
     return outs, fastpath_data
 
-  cpp_pjit_f = xc._xla.pjit(fun, cache_miss, static_argnums)
+  if xc._version < 108:
+    cpp_pjit_f = xc._xla.pjit(fun, cache_miss, static_argnums)  # type: ignore
+  else:
+    cpp_pjit_f = xc._xla.pjit(  # type: ignore
+        getattr(fun, "__name__", "<unnamed function>"),  # type:ignore
+        cache_miss, static_argnums)
 
   return wraps(fun)(cpp_pjit_f)
 
-class _CppPjitAotCall:
-  def __init__(self, fun: Callable):
-    self._fun = fun
-
-  def __call__(self, params: CompiledCallParams):
-
-    def aot_cache_miss(*args, **kwargs):
-      # The first invocation for a signature will use the python path. If it is
-      # a correct signature, the later invocations will use the C++ path.
-      # Otherwise, if the signature is wrong, it will be caught by the python
-      # path during the first invocation.
-      outs, out_flat = stages.Compiled.call(params, *args, **kwargs)
-
-      executable = params.executable
-
-      use_fastpath = (
-          isinstance(executable, pxla.MeshExecutable) and
-          isinstance(executable.unsafe_call, pxla.ExecuteReplicated) and
-          not executable.unsafe_call.has_unordered_effects and
-          not executable.unsafe_call.has_host_callbacks and
-          all(isinstance(x, xc.ArrayImpl) for x in out_flat))
-
-      if use_fastpath:
-        out_avals = [o.aval for o in out_flat]
-        out_committed = [o._committed for o in out_flat]
-        fastpath_data = _PjitFastpathData(executable.xla_executable, params.out_tree,
-                                          executable._in_shardings,
-                                          executable._out_shardings,
-                                          out_avals, out_committed)
-      else:
-        fastpath_data = None
-
-      return outs, fastpath_data
-
-    self._cpp_aot_pjit_f = xc._xla.pjit(self._fun, aot_cache_miss, [])
-    return self._cpp_aot_pjit_f
-
 
 # TODO(yashkatariya): Add pjit microbenchmarks.
 # in_axis_resources and out_axis_resources can't be None as the default value
@@ -366,8 +325,7 @@ def pjit(fun: Callable,
     if kwargs:
       raise NotImplementedError("pjit does not support kwargs")
     if max(static_argnums + donate_argnums, default=-1) >= len(args):
-      raise ValueError(f"jitted function has static_argnums={static_argnums}, "
-                       f"donate_argnums={donate_argnums} but "
+      raise ValueError(f"jitted function has {static_argnums=}, {donate_argnums=} but "
                        f"was called with only {len(args)} positional arguments.")
 
     # Putting this outside of wrapped would make resources lexically scoped
@@ -473,18 +431,15 @@ def pjit(fun: Callable,
         params['resource_env'], params['donated_invars'], params['name'],
         in_is_global, always_lower=True)
 
-    if FLAGS.experimental_cpp_pjit and xc._version >= 96:
-      # This is only used for execution of the compiled object. It is not used
-      # for lowering.
-      create_cpp_call = _CppPjitAotCall(fun)
-    else:
-      create_cpp_call = None
-
     args_kwargs_in_tree = treedef_tuple([in_tree, tree_flatten({})[1]])
     local_in_avals = args_kwargs_in_tree.unflatten(flat_local_in_avals)
     return stages.Lowered.from_flat_info(
-        lowering, args_kwargs_in_tree, local_in_avals, donate_argnums, out_tree,
-        no_kwargs=True, create_cpp_call=create_cpp_call)
+        lowering,
+        args_kwargs_in_tree,
+        local_in_avals,
+        donate_argnums,
+        out_tree,
+        no_kwargs=True)
 
   wrapped.lower = lower
   return wrapped
@@ -658,7 +613,8 @@ def _pjit_jaxpr(fun, out_shardings_thunk, global_in_avals, out_tree):
   try:
     maps._positional_semantics.val = maps._PositionalSemantics.GLOBAL
     with dispatch.log_elapsed_time(f"Finished tracing + transforming {fun.__name__} "
-                                   "for pjit in {elapsed_time} sec"):
+                                   "for pjit in {elapsed_time} sec",
+                                    event=dispatch.JAXPR_TRACE_EVENT):
       jaxpr, global_out_avals, consts = pe.trace_to_jaxpr_dynamic(fun, global_in_avals)
   finally:
     maps._positional_semantics.val = prev_positional_val
@@ -924,6 +880,21 @@ def _resolve_in_shardings(args, pjit_in_shardings, out_shardings, pjit_mesh):
             raise NotImplementedError('Having uncommitted Array sharded on '
                                       'multiple devices is not supported.')
     else:
+      if isinstance(arg, np.ndarray) and not pxla.is_op_sharding_replicated(
+          pjit_in_s._to_xla_op_sharding(arg.ndim)) and xb.process_count() > 1:
+        raise ValueError(
+            'When jax.Array is enabled, passing non-trivial shardings for numpy '
+            'inputs is not allowed. To fix this error, either specify a '
+            'replicated sharding explicitly or use '
+            '`jax.experimental.multihost_utils.host_local_array_to_global_array(...)` '
+            'to convert your host local numpy inputs to a jax.Array which you '
+            'can pass to pjit. '
+            'If the numpy input is the same on each process, then you can use '
+            '`jax.make_array_from_callback(...) to create a `jax.Array` which '
+            'you can pass to pjit. '
+            'Please see the jax.Array migration guide for more information '
+            'https://jax.readthedocs.io/en/latest/jax_array_migration.html#handling-of-host-local-inputs-to-pjit-like-batch-etc. '
+            f'Got arg shape: {arg.shape}, arg value: {arg}')
       if not _is_unspecified(arg_s):
         if committed and not pxla.are_op_shardings_equal(
             pjit_in_s._to_xla_op_sharding(arg.ndim),
@@ -932,7 +903,8 @@ def _resolve_in_shardings(args, pjit_in_shardings, out_shardings, pjit_mesh):
           raise ValueError('Sharding passed to pjit does not match the sharding '
                            'on the respective arg. '
                            f'Got pjit sharding: {op},\n'
-                           f'arg sharding: {arg_s} for arg: {arg}')
+                           f'arg sharding: {arg_s} for arg shape: {arg.shape}, '
+                           f'arg value: {arg}')
       resolved_in_shardings.append(pjit_in_s)
 
   return tuple(resolved_in_shardings)
@@ -1159,17 +1131,20 @@ batching.axis_primitive_batchers[pjit_p] = partial(_pjit_batcher, False, None)
 pxla.spmd_primitive_batchers[pjit_p] = partial(_pjit_batcher, True, None)
 
 def _pjit_batcher_for_sharding(
-    s: OpShardingSharding, dim: int, val: Tuple[str, ...], mesh, ndim: int):
+    s: Union[OpShardingSharding, _UnspecifiedValue], dim: int,
+    val: Tuple[str, ...], mesh, ndim: int):
+  if _is_unspecified(s):
+    return s
   if not val:
-    new_op = s._op_sharding.clone()
+    new_op = s._op_sharding.clone()  # type: ignore
     tad = list(new_op.tile_assignment_dimensions)
     tad.insert(dim, 1)
     new_op.tile_assignment_dimensions = tad
-    return OpShardingSharding(s._device_assignment, new_op)
+    return OpShardingSharding(s._device_assignment, new_op)  # type: ignore
   else:
     assert isinstance(s, OpShardingSharding)
     assert not mesh.empty
-    parsed_pspec = parse_flatten_op_sharding(s._op_sharding, mesh)[0]
+    parsed_pspec = parse_flatten_op_sharding(s._op_sharding, mesh)[0]  # type: ignore
     parsed_pspec = parsed_pspec.insert_axis_partitions(dim, val)
     mps = NamedSharding._from_parsed_pspec(mesh, parsed_pspec)
     return OpShardingSharding(mps._device_assignment, mps._to_xla_op_sharding(ndim))
