@@ -22,8 +22,7 @@ import itertools as it
 import operator
 import re
 from typing import (Any, Callable, Dict, List, NamedTuple, Optional,
-                    Sequence, Set, Type, Tuple, Union)
-from typing_extensions import Protocol
+                    Protocol, Sequence, Set, Type, Tuple, Union)
 
 import numpy as np
 
@@ -32,7 +31,7 @@ from jax import core
 from jax._src import device_array
 from jax._src import dtypes
 from jax._src import source_info_util
-from jax._src.abstract_arrays import (make_shaped_array, array_types)
+from jax._src.abstract_arrays import numpy_scalar_types
 from jax.core import (ConcreteArray, ShapedArray, str_eqn_compact)
 import jax._src.pretty_printer as pp
 from jax._src.util import (prod, new_name_stack, safe_zip, safe_map,
@@ -247,8 +246,12 @@ def canonicalize_dtype(x):
     return canonicalize_dtype(x.__jax_array__())
   raise TypeError(f"No canonicalize_dtype handler for type: {type(x)}")
 
+
 def _canonicalize_ndarray_dtype(x):
-  return np.asarray(x, dtypes.canonicalize_dtype(dtypes.result_type(x)))
+  return np.asarray(x, dtypes.canonicalize_dtype(x.dtype))
+
+def _canonicalize_numpy_scalar_dtype(x):
+  return np.asarray(x, dtypes.canonicalize_dtype(np.dtype(x)))
 
 def _canonicalize_python_scalar_dtype(typ, x):
   return np.asarray(
@@ -258,7 +261,8 @@ canonicalize_dtype_handlers: Dict[Any, Callable] = {}
 for t in device_array.device_array_types:
   canonicalize_dtype_handlers[t] = identity
 canonicalize_dtype_handlers.update(
-    (t, _canonicalize_ndarray_dtype) for t in array_types)
+    (t, _canonicalize_ndarray_dtype) for t in numpy_scalar_types)
+canonicalize_dtype_handlers[np.ndarray] = _canonicalize_ndarray_dtype
 canonicalize_dtype_handlers.update(
     (t, partial(_canonicalize_python_scalar_dtype, t)) for t in _scalar_types)
 canonicalize_dtype_handlers[core.Token] = identity
@@ -281,12 +285,25 @@ def _make_abstract_python_scalar(typ, val):
   return ShapedArray((), dtypes._scalar_type_to_dtype(typ, val),
                      weak_type=typ is not bool)
 
+def _make_shaped_array_for_numpy_scalar(x: np.generic) -> ShapedArray:
+  dtype = np.dtype(x)
+  dtypes.check_valid_dtype(dtype)
+  return ShapedArray(np.shape(x), dtypes.canonicalize_dtype(dtype))
+
+def _make_shaped_array_for_numpy_array(x: np.ndarray) -> ShapedArray:
+  dtype = x.dtype
+  dtypes.check_valid_dtype(dtype)
+  return ShapedArray(x.shape, dtypes.canonicalize_dtype(dtype))
+
+
 pytype_aval_mappings: Dict[Any, Callable[[Any], core.AbstractValue]] = {}
 for t in device_array.device_array_types:
   pytype_aval_mappings[t] = operator.attrgetter('aval')
 pytype_aval_mappings[core.DArray] = operator.attrgetter('_aval')
 pytype_aval_mappings[core.Token] = lambda _: core.abstract_token
-pytype_aval_mappings.update((t, make_shaped_array) for t in array_types)
+pytype_aval_mappings.update((t, _make_shaped_array_for_numpy_scalar)
+                            for t in numpy_scalar_types)
+pytype_aval_mappings[np.ndarray] = _make_shaped_array_for_numpy_array
 pytype_aval_mappings.update(
     (t, partial(_make_abstract_python_scalar, t)) for t in _scalar_types)
 
